#!/usr/bin/perl
# Eenos exim perl startups
# VERSION=1.1
use strict;
use warnings;
sub get_env
{
    my ($envvar) = @_;
    if ($envvar eq "" ) { return ""; };
    return $ENV{$envvar};
}

# Ready any  valid yaml file 
sub readyaml
{
    use YAML::XS 'LoadFile';       
    if (my ($datafile) = LoadFile(@_)){
        return $datafile;
    }else{
        return;
    }
    #use Data::Dumper;
    #my ($datafile) = LoadFile(@_);
    #return $datafile;
    #my $data=Dumper($datafile);
    #print $data;
    #return $data;
}

sub get_domain_owner
{
    my ($domain) = @_;
    my $username="";
    my $userdomains=readyaml('/etc/userdomains');
    if (my $user=$userdomains ->{$domain} ){
        return $user;    
    }else{
        return -1;
    }      
}

sub safe_name
{
    my ($name) =  @_;
    if ($name =~ /\//)
    {
        return 0;
    }

    if ($name =~ /\\/)
    {
        return 0;
    }

    if ($name =~ /\|/)
    {
        return 0;
    }

    if ($name =~ /\.\./)
    {
        return 0;
    }

    return 1;
}

sub find_uid_auth_id
{
    # this  'username' or 'user@domain.com'
    my ($auth_id) = @_;
    my $unixuser = 1;
    my $domain = "";
    my $user = "";
    my $username = $auth_id;
    my @pw;

    if (!safe_name($username))
    {
        Exim::log_write("find_uid_auth_id: Invalid username: $username");
        return "-1";
    }

    if ($auth_id =~ /\@/)
    {
        $unixuser = 0;
        ($user,$domain) = split(/\@/, $auth_id);
        if ($domain eq "") { return "-1"; }
    }

    if (!$unixuser)
    {
        # Find the owner of domain from eenos user domains file
        my $thisuer = get_domain_owner($domain);        
        if ($thisuer ne "-1" )
        {
            $username = $thisuer;
        }
    }
    if ( (@pw = getpwnam($username))  )
    {
        return $pw[2];
    }

    return -1;
}

sub find_uid_httpd
{
    my ($work_path) = @_;
    my @pw;    
    my @dirs = split(/\//, $work_path);
    foreach my $dir (@dirs) {
    # Check the script dir with user home dir for validation
        if ( (@pw = getpwnam($dir))  )
        {
            if ($work_path =~/^$pw[7]/)
            {
                    return $pw[2];
            }
        }
    }
    return -1;
}

sub find_uid_sender
{
    if (my $sender_address = Exim::expand_string('$sender_address')){    
        my ($user,$domain) = split(/\@/, $sender_address);
        #Exim::log_write("sender address : $sender_address");
        my $primary_hostname = Exim::expand_string('$primary_hostname');
        if ( $domain eq $primary_hostname )
        {
            my @pw = getpwnam($user);
            return $pw[2];
        }

        my $username = get_domain_owner($domain);

        if ( (my @pw = getpwnam($username))  )
        {
                return $pw[2];
        }
        return -1;
    }else{
        return -1;
    }
    
}



sub find_uid
{
    my $uid = Exim::expand_string('$originator_uid');
    my $username = getpwuid($uid);
    my $auth_id = Exim::expand_string('$authenticated_id');
    my $work_path = $ENV{'PWD'};
    if ($username eq "apache" || $username eq "nobody" || $username eq "www-data")
    {
        my $httpd_uid = find_uid_httpd($work_path);
        if ($httpd_uid != -1) { return $httpd_uid; }
    }

    if ($username ne "" && -f "/var/eenos/users/$username" )
    {
        return $uid;
    }

    my $auth_uid = find_uid_auth_id($auth_id);
    if ($auth_uid != -1) { return $auth_uid; }

    # we don't want to rely on this, but it's all thats left.
    return find_uid_sender;
}

#Find user name from uid 
sub get_username
{
    my ($uid) = @_; 
    if ($uid == -1) { return "unknown" };
    my $name = getpwuid($uid);
    return $name;
}

sub is_suspended
{
    my ($domain,$user) = @_ ;    
    my $primary_hostname = Exim::expand_string('$primary_hostname');
    #my $sender_address = Exim::expand_string('$sender_address');
    #Exim::log_write("sender : $domain , $user");
    #my ($user,$domain) = split(/\@/, $sender_address);
    my $result="no";
    my $euid=find_uid();
    my $euser=get_username($euid);
    my $mid                 = Exim::expand_string('$message_id');
    if ( -f "/var/eenos/suspended/$euser" ){
        Exim::log_write("** R=$mid Eenos account is suspened : $euser");
        $result="yes";
        return $result;
    }
    if ( $domain eq $primary_hostname )
    {
        if ( -f "/var/eenos/suspended/$user" ){
            Exim::log_write("** R=$mid Recipient account is suspened : $user");
            $result="yes";
        }else{
            $result="no";
        } 
    }else{
        my $userdomains=readyaml('/etc/userdomains');
        if (my $user=$userdomains ->{$domain} ){
            if ( -f "/var/eenos/suspended/$user" ){
                Exim::log_write("** R=$mid Recipient account is suspened : $user");
                $result="yes";
            }else{
                $result="no";
            } 
        }else{
            $result="no";
        }     
    } 
    #Exim::log_write("is suspend  : $result");
    return $result;
}

# Find SNI SSL certificate for domain
sub find_domain_sni_crt
{
    my $default_crt='/etc/exim/exim.crt.pem';
    my $crt=$default_crt;
    if (my ($domain) = @_ ){
        #print $domain;
        my $primary_hostname = Exim::expand_string('$primary_hostname');        
        if ($domain eq $primary_hostname){
            $crt =$default_crt;
        }else{
            my $domcrt = "/var/eenos/ssl/exim/$domain.crt";
            my $domkey ="/var/eenos/ssl/exim/$domain.key";
            if ( ( -f $domcrt) &&  ( -f $domkey) ){
                $crt=$domcrt;
            }else{                
                if($domain  =~ /^(mail|imap|smtp|www)\./ ){
                    #print "$domain :  Start with subdomain \n";
                    my($first, $last) = split(/\./, $domain,2);
                    # print "$first\n";
                    # print "$last\n";
                    my $domcrt = "/var/eenos/ssl/exim/$last.crt";
                    my $domkey ="/var/eenos/ssl/exim/$last.key";
                    if ( ( -f $domcrt) &&  ( -f $domkey) ){
                        $crt=$domcrt;
                    }else{
                        $crt=$default_crt;
                    }
                }else{
                    $crt=$default_crt;
                }                
            }       
        }        
    }else{
        $crt='/etc/exim/exim.crt.pem';
    }
    return $crt;
}

# Find SNI SSL key for domain
sub find_domain_sni_key
{   my $default_key ="/etc/exim/exim.key.pem";
    my $key=$default_key;
    if (my ($domain) = @_ ){
        #print $domain;
        my $primary_hostname = Exim::expand_string('$primary_hostname');        
        if ($domain eq $primary_hostname){
            $key=$default_key;
        }else{
            my $domcrt = "/var/eenos/ssl/exim/$domain.crt";
            my $domkey ="/var/eenos/ssl/exim/$domain.key";
            if ( ( -f $domcrt) &&  ( -f $domkey) ){
                $key=$domkey;
            }else{
                if($domain  =~ /^(mail|imap|smtp|www)\./ ){
                    # print "$domain :  Start with subdomain \n";
                    my($first, $last) = split(/\./, $domain,2);
                    # print "$first\n";
                    # print "$last\n";
                    my $domcrt = "/var/eenos/ssl/exim/$last.crt";
                    my $domkey ="/var/eenos/ssl/exim/$last.key";
                    if ( ( -f $domcrt) &&  ( -f $domkey) ){
                        $key=$domkey;
                    }else{
                        $key=$default_key;  
                    }
                }else{
                    $key=$default_key;
                }                
            }       
        }        
    }else{
        $key=$default_key;
    }
    return $key;
}

sub have_spamd
{
    if ( -f '/var/run/spamd.sock'){
        return 1;
    }else{
        return -1;
    }
}


sub track_this_email {
    my $primary_hostname ||= Exim::expand_string('$primary_hostname');
    my $original_domain       = Exim::expand_string('$original_domain');
    my $sender_address_domain = Exim::expand_string('$sender_address_domain');
    my $originator_uid        = Exim::expand_string('$originator_uid');
    my $originator_gid        = Exim::expand_string('$originator_gid');
    my $caller_uid            = Exim::expand_string('$caller_uid');
    my $caller_gid            = Exim::expand_string('$caller_gid');
    my $auth_id               = find_uid() ;
    my $headers =
        "X-AntiAbuse: This header was added to track abuse, please include it with any abuse report\n"
      . "X-AntiAbuse: Primary Hostname - $primary_hostname\n"
      . "X-AntiAbuse: Original Domain - $original_domain\n"
      . "X-AntiAbuse: Originator/Caller UID/GID - [$originator_uid $originator_gid] / [$caller_uid $caller_gid]\n"
      . "X-AntiAbuse: Auth UID - [$auth_id]\n"
      . "X-AntiAbuse: Sender Address Domain - $sender_address_domain\n";      
    return ($headers);

}

sub have_domain_dkim
{
   my $sender_address_domain = Exim::expand_string('$sender_address_domain');
   if ( -f "/var/eenos/domain_keys/private/$sender_address_domain" ){
       return 1;
   } else{
       return -1;
   }
}

sub untaint {
    return $_[0];
}

# Add your custom perl statups
if ( -e "/etc/exim/exim.custom.pl" ) {
	do '/etc/exim/exim.custom.pl';
}

sub have_distribution
{  
    my ($domain) = @_ ;
    #my $original_domain       = Exim::expand_string('$original_domain');
    my $original_domain = $domain;
    my $primary_hostname = Exim::expand_string('$primary_hostname'); 

    if ($domain eq $primary_hostname ){
        return -1;
    }else{
        my $user = get_domain_owner($original_domain);
        my $local_part = Exim::expand_string('$local_part'); 
        #my $sender_address_domain = Exim::expand_string('$sender_address_domain');
        #Exim::log_write("** checking distribution original: $original_domain, send : $sender_address_domain, user: $user, domain: $domain");    
        if ( $user ne "-1" )    {
            my $distfile = "/var/eenos/userdata/$user/mail/distribute/$original_domain" ;
            my $email = $local_part.'@'.$domain;
            if ( -f "$distfile" ){
                if ( -z $distfile ){
                    # The distribution list is empty so no distribution
                    return -1;
                }else{
                    my $distdata = readyaml($distfile);
                    if (my $list = $distdata ->{$email} ){                     
                        #Exim::log_write("=> Processing mail distribution from  $email => $list");
                        return 1;
                    }else{
                        return -1;
                    }
                }
            } else{
                # There is no distribution file so no distribution
                return -1;
            }
        } else {
            # There is no user so no distribution    
            return -1;
        }
    }
}

sub get_distribution
{
    my ($domain) = @_ ;
    my $user = get_domain_owner($domain);
    if ( $user ne "-1" )    {
        my $local_part = Exim::expand_string('$local_part');
        my $email = $local_part.'@'.$domain;
        my $distfile = "/var/eenos/userdata/$user/mail/distribute/$domain" ;
        if ( -f "$distfile" ){
                if ( -z $distfile ){
                    # The distribution list is empty so no distribution
                    return ':blackhole:';
                }else{
                    my $distdata = readyaml($distfile);
                    if (my $list = $distdata ->{$email} ){                     
                        #Exim::log_write("=> Processing mail distribution from  $email => $list");
                        return $list;
                    }else{                         
                        return ":blackhole:";
                    }
                }
        } else{                
            return ":blackhole:";
        }
        
    } else {
        #Exim::log_write("=> NO USER ");
        return ":blackhole:";
    }
}

sub have_pipemail
{  
    my ($domain) = @_ ;
    #my $original_domain       = Exim::expand_string('$original_domain');
    my $original_domain = $domain;
    my $primary_hostname = Exim::expand_string('$primary_hostname'); 

    if ($domain eq $primary_hostname ){
        return -1;
    }else{
        my $user = get_domain_owner($original_domain);
        my $local_part = Exim::expand_string('$local_part'); 
        #my $sender_address_domain = Exim::expand_string('$sender_address_domain');
        #Exim::log_write("** checking distribution original: $original_domain, send : $sender_address_domain, user: $user, domain: $domain");    
        if ( $user ne "-1" )    {
            my $distfile = "/var/eenos/userdata/$user/mail/pipe/$original_domain" ;
            my $email = $local_part.'@'.$domain;
            if ( -f "$distfile" ){
                if ( -z $distfile ){
                    # The distribution list is empty so no distribution
                    return -1;
                }else{
                    my $distdata = readyaml($distfile);
                    if (my $list = $distdata ->{$email} ){                     
                        #Exim::log_write("=> Piping mail  from  $email => $list");
                        return 1;
                    }else{
                        return -1;
                    }
                }
            } else{
                # There is no distribution file so no distribution
                return -1;
            }
        } else {
            # There is no user so no distribution    
            return -1;
        }
    }
}
sub get_pipemail
{
    my ($domain) = @_ ;
    my $user = get_domain_owner($domain);
    if ( $user ne "-1" )    {
        my $local_part = Exim::expand_string('$local_part');
        my $email = $local_part.'@'.$domain;
        my $distfile = "/var/eenos/userdata/$user/mail/pipe/$domain" ;
        if ( -f "$distfile" ){
                if ( -z $distfile ){
                    # The distribution list is empty so no distribution
                    return ':blackhole:';
                }else{
                    my $distdata = readyaml($distfile);
                    if (my $list = $distdata ->{$email} ){                     
                        #Exim::log_write("=> Processing Pipe  from  $email => $list");
                        return $list;
                    }else{                         
                        return ":blackhole:";
                    }
                }
        } else{                
            return ":blackhole:";
        }
        
    } else {
        #Exim::log_write("=> NO USER ");
        return ":blackhole:";
    }
}

# Dmain filter
sub have_domainfilter
{  
    my ($domain) = @_ ;    
    my $primary_hostname = Exim::expand_string('$primary_hostname'); 
    if ($domain eq $primary_hostname ){
        # No filter for hostname
        return -1;
    }else{
        my $user = get_domain_owner($domain);        
        if ( $user ne "-1" )    {
            my $filter = "/var/eenos/userdata/$user/mail/domainfilter/$domain" ;            
            if ( -f "$filter" ){
                if ( -z $filter ){
                    # The filter is empty
                    return -1;
                }else{
                    # Filter exist for domain and is not empty
                    return 1;                   
                }
            } else{
                # There is no domain filter file
                return -1;
            }
        } else {
            # There is no user so no filter
            return -1;
        }
    }
}
sub get_domainfilter
{
    my ($domain) = @_ ;
    my $user = get_domain_owner($domain);
    if ( $user ne "-1" )    {       
        my $filter = "/var/eenos/userdata/$user/mail/domainfilter/$domain" ;
        if ( -f "$filter" ){
                if ( -z $filter ){
                    # The filter is empty so no 
                    return ':blackhole:';
                }else{
                    # Exim::log_write("=> Processing Domain Filter  $domain => $filter");
                    return $filter;                    
                }
        } else{                
            return ":blackhole:";
        }        
    } else {        
        return ":blackhole:";
    }
}
 # Filter for email account
sub have_emailfilter
{  
    my ($domain) = @_ ;    
    my $original_domain = $domain;
    my $primary_hostname = Exim::expand_string('$primary_hostname'); 
    if ($domain eq $primary_hostname ){
        return -1;
    }else{
        my $user = get_domain_owner($original_domain);
        my $local_part = Exim::expand_string('$local_part');            
        if ( $user ne "-1" )    {
            my $emailfilter = "/var/eenos/userdata/$user/mail/accountfilter/$original_domain/$local_part" ;
            my $email = $local_part.'@'.$domain;
            if ( -f "$emailfilter" ){
                if ( -z $emailfilter ){
                    # User filter  is empty so no distribution
                    return -1;
                }else{
                    # filter exists
                    return 1;
                }
            } else{
                # There is no filetr file 
                return -1;
            }
        } else {
            # There is no user     
            return -1;
        }
    }
}
sub get_emailfilter
{
    my ($domain) = @_ ;
    my $user = get_domain_owner($domain);
    if ( $user ne "-1" )    {
        my $local_part = Exim::expand_string('$local_part');
        my $email = $local_part.'@'.$domain;
        my $emailfilter = "/var/eenos/userdata/$user/mail/accountfilter/$domain/$local_part" ;       
        if ( -f "$emailfilter" ){
                if ( -z $emailfilter ){
                    # The filter is empty 
                    return ':blackhole:';
                }else{
                    #Exim::log_write("=> Processing Account Filter  $email => $emailfilter");
                    return $emailfilter;
                }
        } else{                
            return ":blackhole:";
        }
        
    } else {
        #Exim::log_write("=> NO USER ");
        return ":blackhole:";
    }
}

sub have_forwardemail
{  
    my ($domain) = @_ ;
    #my $original_domain       = Exim::expand_string('$original_domain');
    my $original_domain = $domain;
    my $primary_hostname = Exim::expand_string('$primary_hostname'); 

    if ($domain eq $primary_hostname ){
        return -1;
    }else{
        my $user = get_domain_owner($original_domain);
        my $local_part = Exim::expand_string('$local_part'); 
        #my $sender_address_domain = Exim::expand_string('$sender_address_domain');
        #Exim::log_write("** checking distribution original: $original_domain, send : $sender_address_domain, user: $user, domain: $domain");    
        if ( $user ne "-1" )    {
            my $distfile = "/var/eenos/userdata/$user/mail/forwarder/$original_domain" ;
            my $email = $local_part.'@'.$domain;
            if ( -f "$distfile" ){
                if ( -z $distfile ){
                    # The forward list is empty so no forward
                    return -1;
                }else{
                    my $distdata = readyaml($distfile);
                    if (my $list = $distdata ->{$email} ){                     
                        #Exim::log_write("=> Forwarding mail  from  $email => $list");
                        return 1;
                    }else{
                        return -1;
                    }
                }
            } else{
                # There is no forwarder file so no forward
                return -1;
            }
        } else {
            # There is no user so no forward    
            return -1;
        }
    }
}
sub get_forwardmail
{
    my ($domain) = @_ ;
    my $user = get_domain_owner($domain);
    if ( $user ne "-1" )    {
        my $local_part = Exim::expand_string('$local_part');
        my $email = $local_part.'@'.$domain;
        my $distfile = "/var/eenos/userdata/$user/mail/forwarder/$domain" ;
        if ( -f "$distfile" ){
                if ( -z $distfile ){
                    # The forward list is empty so no forward
                    return ':blackhole:';
                }else{
                    my $distdata = readyaml($distfile);
                    if (my $list = $distdata ->{$email} ){                     
                        #Exim::log_write("=> Processing forward  from  $email => $list");
                        return $list;
                    }else{                         
                        return ":blackhole:";
                    }
                }
        } else{                
            return ":blackhole:";
        }
        
    } else {
        #Exim::log_write("=> NO USER ");
        return ":blackhole:";
    }
}

1;